@@ -2,6 +2,7 @@
 #
 # This file is part of Invenio.
 # Copyright (C) 2015-2018 CERN.
+# Copyright (C)      2021 TU Wien.
 #
 # Invenio is free software; you can redistribute it and/or modify it
 # under the terms of the MIT License; see LICENSE file for more details.
@@ -54,6 +55,8 @@ keys:
 - ``authorized_handler`` - Import path to authorized callback handler.
 - ``disconnect_handler`` - Import path to disconnect callback handler.
 - ``signup_handler`` - A dictionary of import path to sign up callback handler.
+- ``precedence_mask`` - A mask determining which user info values should
+  override user input during sign-up.
 
 .. code-block:: python
 
@@ -69,6 +72,99 @@ keys:
                 setup="...",
                 view="...",
             ),
+            precedence_mask=dict(
+                email=True
+            ),
+            params=dict(...),
+            )
+        )
+    )
+
+Note on the ``precedence_mask``:
+
+This mask is used during sign-up of new users via external OAuth
+providers, to determine for which of the new user's properties the
+`user_info` given by the OAuth provider must take precedence over
+any user input.
+
+Properties marked with `False` in the precedence mask will be used
+as specified by the user.
+Any properties marked with `True` (or not appearing) in the precedence
+mask will be taken from the OAuth service's `user_info` dictionary,
+overriding any potential user input from registration forms.
+If a property is missing from the `user_info` dictionary, the
+configured value in the precedence mask is irrelevant.
+
+For instance, if the following user info were given by the OAuth
+remote app during signup, and the precedence mask configured as
+follows:
+
+.. code-block:: python
+
+    # user_info from the oauth remote app
+    {
+        "email": "user@inveniosoftware.org",
+        "password": "somepassword",
+        "profile": {
+            "username": "test-user",
+            "full_name": "Test User",
+        }
+    }
+
+    # precedence_mask
+    {
+        "email": True,
+        "profile": {
+            "username": True,
+            "full_name": False,
+        }
+    }
+
+Then, the values for `email` and `profile.username` from the
+`user_info` would be overriding any user input from the registration
+form.
+All other values would be used as provided by the user in the form.
+
+WARNING: Allowing users to specify their email address arbitrarily
+(`precedence_mask["email"] = False`) during sign-up may have severe
+security implications, as the linking of external accounts with
+accounts in Invenio is done by matching email addresses!
+
+
+Remote REST application
+^^^^^^^^^^^^^^^^^^^^^^^
+Configuration of a single remote REST application is a dictionary with the
+same keys as Remote application in addition to:
+
+- ``response_handler`` - Import path to response callback handler.
+- ``authorized_redirect_url`` - URL path to redirect your SPA when login was
+  successfull.
+- ``disconnect_redirect_url`` - URL path to redirect your SPA after logging
+  out.
+- ``signup_redirect_url`` - URL path to redirect your SPA to sign up a user.
+- ``error_redirect_url`` - URL path to redirect your SPA when an error ocurred.
+
+
+.. code-block:: python
+
+    OAUTHCLIENT_REMOTE_APPS = dict(
+        myapp=dict(
+            title='...',
+            description='...',
+            icon='...',
+            authorized_handler="...",
+            disconnect_handler="...",
+            signup_handler=dict(
+                info="...",
+                setup="...",
+                view="...",
+            ),
+            response_handler=("..."),
+            authorized_redirect_url="...",
+            disconnect_redirect_url="...",
+            signup_redirect_url="...",
+            error_redirect_url="...",
+            precedence_mask=dict(...),
             params=dict(...),
             )
         )
@@ -197,6 +293,10 @@ setting ``remote_app`` in your remote application configuration.
 
 """
 
+from invenio_oauthclient.utils import _create_registrationform
+
+from .views.client import auto_redirect_login
+
 OAUTHCLIENT_REMOTE_APPS = {}
 """Configuration of remote applications."""
 
@@ -209,5 +309,26 @@ OAUTHCLIENT_STATE_EXPIRES = 300
 OAUTHCLIENT_STATE_ENABLED = True
 """Internal variable used to disable state validation during tests."""
 
+OAUTHCLIENT_SIGNUP_FORM = _create_registrationform
+"""Function called to render the sign up form after authorization succeeded."""
+
 OAUTHCLIENT_SIGNUP_TEMPLATE = 'invenio_oauthclient/signup.html'
 """Template for the signup page."""
+
+OAUTHCLIENT_REST_REMOTE_APPS = {}
+"""Configuration of remote rest applications."""
+
+OAUTHCLIENT_REST_DEFAULT_ERROR_REDIRECT_URL = '/'
+"""Configuration of default error redirect URL."""
+
+OAUTHCLIENT_REST_DEFAULT_RESPONSE_HANDLER = None
+"""Default REST response handler."""
+
+OAUTHCLIENT_AUTO_REDIRECT_TO_EXTERNAL_LOGIN = False
+"""Redirect to the only external login service under specific conditions.
+
+If this option is enabled and there is exactly one external authentication
+service enabled (i.e. one OAuthClient remote app is configured, and local
+login is disabled), the login view function will automatically redirect to
+this external authentication service.
+"""
